@@ -23,9 +23,9 @@
  *
  * Contributor(s): 
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21293 $
+ * $Author: rjongbloed $
+ * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
  */
 
 #ifndef OPAL_CODEC_VIDCODEC_H
@@ -144,12 +144,16 @@ class OpalVideoTranscoder : public OpalTranscoder
 #if OPAL_STATISTICS
     virtual void GetStatistics(OpalMediaStatistics & statistics) const;
 #endif
+
+    bool WasLastFrameIFrame() const { return lastFrameWasIFrame; }
+    void ForceIFrame()              { forceIFrame = true; }
   //@}
 
   protected:
     PINDEX inDataSize;
     PINDEX outDataSize;
     bool   forceIFrame;
+    bool   lastFrameWasIFrame;
 
 #if OPAL_STATISTICS
     DWORD m_totalFrames;
