@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21231 $
- * $Author: rjongbloed $
- * $Date: 2008-10-03 11:48:42 +0000 (Fri, 03 Oct 2008) $
+ * $Revision: 21715 $
+ * $Author: hfriederich $
+ * $Date: 2008-12-08 23:17:16 +0000 (Mon, 08 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -573,6 +573,15 @@ PBoolean H245NegLogicalChannel::OpenWhileLocked(const H323Capability & capabilit
            << ", channel->OnSendingPDU() failed");
     return PFalse;
   }
+  
+  // ensure the correct (external) session ID is used
+  if (open.HasOptionalField(H245_OpenLogicalChannel::e_reverseLogicalChannelParameters)) {
+    H245_H2250LogicalChannelParameters & param = open.m_reverseLogicalChannelParameters.m_multiplexParameters;
+    param.m_sessionID = connection.GetExternalSessionID(sessionID, capability.GetMediaFormat().GetMediaType());
+  } else {
+    H245_H2250LogicalChannelParameters & param = open.m_forwardLogicalChannelParameters.m_multiplexParameters;
+    param.m_sessionID = connection.GetExternalSessionID(sessionID, capability.GetMediaFormat().GetMediaType());
+  }
 
   if (replacementFor > 0) {
     if (open.HasOptionalField(H245_OpenLogicalChannel::e_reverseLogicalChannelParameters)) {
