@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21864 $
+ * $Revision: 22178 $
  * $Author: rjongbloed $
- * $Date: 2008-12-23 04:19:19 +0000 (Tue, 23 Dec 2008) $
+ * $Date: 2009-03-09 10:04:21 +0000 (Mon, 09 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -132,6 +132,7 @@ ReasonToSIPCode[] = {
 // This table comes from RFC 3398 para 8.2.6.1
 //
 SIPCodeToReason[] = {
+  { SIP_PDU::Failure_RequestTerminated          , OpalConnection::EndedByNoAnswer          ,  19 }, // No answer
   { SIP_PDU::Failure_BadRequest                 , OpalConnection::EndedByQ931Cause         ,  41 }, // Temporary Failure
   { SIP_PDU::Failure_UnAuthorised               , OpalConnection::EndedBySecurityDenial    ,  21 }, // Call rejected (*)
   { SIP_PDU::Failure_PaymentRequired            , OpalConnection::EndedByQ931Cause         ,  21 }, // Call rejected
@@ -193,8 +194,6 @@ SIPConnection::SIPConnection(OpalCall & call,
 {
   synchronousOnRelease = false;
 
-  m_dialog.SetCallID(token);
-
   SIPURL adjustedDestination = destination;
   // Look for a "proxy" parameter to override default proxy
   PStringToString params = adjustedDestination.GetParamVars();
@@ -255,6 +254,9 @@ SIPConnection::~SIPConnection()
   delete authentication;
   delete originalInvite;
 
+  if (transport != NULL)
+    transport->CloseWait();
+
   if (deleteTransport)
     delete transport;
 
@@ -276,61 +278,64 @@ void SIPConnection::OnReleased()
   SetPhase(ReleasingPhase);
 
   PSafePtr<SIPTransaction> byeTransaction;
+  SIPDialogNotification::Events notifyDialogEvent = SIPDialogNotification::NoEvent;
+  SIP_PDU::StatusCodes sipCode = SIP_PDU::IllegalStatusCode;
 
   switch (releaseMethod) {
     case ReleaseWithNothing :
       if (!forkedInvitations.IsEmpty())
-        NotifyDialogState(SIPDialogNotification::Terminated, SIPDialogNotification::Timeout);
+        notifyDialogEvent = SIPDialogNotification::Timeout;
       break;
 
     case ReleaseWithResponse :
-      {
-        SIP_PDU::StatusCodes sipCode = SIP_PDU::Failure_BadGateway;
-
-        // Try find best match for return code
-        for (PINDEX i = 0; i < PARRAYSIZE(ReasonToSIPCode); i++) {
-          if (ReasonToSIPCode[i].q931Cause == GetQ931Cause()) {
-            sipCode = ReasonToSIPCode[i].code;
-            break;
-          }
-          if (ReasonToSIPCode[i].reason == callEndReason) {
-            sipCode = ReasonToSIPCode[i].code;
-            break;
-          }
+      // Try find best match for return code
+      sipCode = SIP_PDU::Failure_BadGateway;
+      for (PINDEX i = 0; i < PARRAYSIZE(ReasonToSIPCode); i++) {
+        if (ReasonToSIPCode[i].q931Cause == GetQ931Cause()) {
+          sipCode = ReasonToSIPCode[i].code;
+          break;
+        }
+        if (ReasonToSIPCode[i].reason == callEndReason) {
+          sipCode = ReasonToSIPCode[i].code;
+          break;
         }
-
-        // EndedByCallForwarded is a special case because it needs extra paramater
-        SendInviteResponse(sipCode, NULL, callEndReason == EndedByCallForwarded ? (const char *)forwardParty : NULL);
-        NotifyDialogState(SIPDialogNotification::Terminated, SIPDialogNotification::Rejected, sipCode);
       }
+
+      // EndedByCallForwarded is a special case because it needs extra paramater
+      SendInviteResponse(sipCode, NULL, callEndReason == EndedByCallForwarded ? (const char *)forwardParty : NULL);
+      notifyDialogEvent = SIPDialogNotification::Rejected;
       break;
 
     case ReleaseWithBYE :
       // create BYE now & delete it later to prevent memory access errors
       byeTransaction = new SIPTransaction(*this, *transport, SIP_PDU::Method_BYE);
-      switch (GetCallEndReason()) {
-        case EndedByRemoteUser :
-          NotifyDialogState(SIPDialogNotification::Terminated, SIPDialogNotification::RemoteBye);
-          break;
-
-        case OpalConnection::EndedByCallForwarded :
-          NotifyDialogState(SIPDialogNotification::Terminated, SIPDialogNotification::Replaced);
-          break;
-
-        default :
-          NotifyDialogState(SIPDialogNotification::Terminated, SIPDialogNotification::LocalBye);
-      }
       break;
 
     case ReleaseWithCANCEL :
-      {
-        PTRACE(3, "SIP\tCancelling " << forkedInvitations.GetSize() << " transactions.");
-        for (PSafePtr<SIPTransaction> invitation(forkedInvitations, PSafeReference); invitation != NULL; ++invitation)
-          invitation->Cancel();
-      }
-      NotifyDialogState(SIPDialogNotification::Terminated, SIPDialogNotification::Cancelled);
+      PTRACE(3, "SIP\tCancelling " << forkedInvitations.GetSize() << " transactions.");
+      for (PSafePtr<SIPTransaction> invitation(forkedInvitations, PSafeReference); invitation != NULL; ++invitation)
+        invitation->Cancel();
+      notifyDialogEvent = SIPDialogNotification::Cancelled;
   }
 
+  // No termination event set yet, get it from the call end reason
+  if (notifyDialogEvent == SIPDialogNotification::NoEvent) {
+    switch (GetCallEndReason()) {
+      case EndedByRemoteUser :
+        notifyDialogEvent = SIPDialogNotification::RemoteBye;
+        break;
+
+      case OpalConnection::EndedByCallForwarded :
+        notifyDialogEvent = SIPDialogNotification::Replaced;
+        break;
+
+      default :
+        notifyDialogEvent = SIPDialogNotification::LocalBye;
+    }
+  }
+
+  NotifyDialogState(SIPDialogNotification::Terminated, notifyDialogEvent, sipCode);
+
   // Close media
   CloseMediaStreams();
 
@@ -353,9 +358,6 @@ void SIPConnection::OnReleased()
   SetPhase(ReleasedPhase);
 
   OpalRTPConnection::OnReleased();
-
-  if (transport != NULL)
-    transport->CloseWait();
 }
 
 
@@ -379,7 +381,7 @@ bool SIPConnection::TransferConnection(const PString & remoteParty)
     if (sip != NULL) {
       PStringStream referTo;
       referTo << sip->GetRemotePartyURL()
-              << "?Replaces=" << sip->GetToken()
+              << "?Replaces=" << sip->GetDialog().GetCallID()
               << "%3Bto-tag%3D"   << sip->GetDialog().GetLocalTag() // "to/from" is from the other sides perspective
               << "%3Bfrom-tag%3D" << sip->GetDialog().GetRemoteTag();
       referTransaction = new SIPRefer(*this, *transport, referTo, localPartyURL);
@@ -406,7 +408,7 @@ PBoolean SIPConnection::SetAlerting(const PString & /*calleeName*/, PBoolean wit
   
   PTRACE(3, "SIP\tSetAlerting");
 
-  if (GetPhase() != SetUpPhase) 
+  if (GetPhase() >= AlertingPhase) 
     return PFalse;
 
   if (!withMedia) 
@@ -600,7 +602,7 @@ static void SetNxECapabilities(SDPMediaDescription * localMedia,
   OpalMediaFormat adjustedFormat = mediaFormat;
   adjustedFormat.SetPayloadType(nxePayloadCode);
   adjustedFormat.SetOptionString("FMTP", handler->GetRxCapability());
-  localMedia->AddSDPMediaFormat(new SDPMediaFormat(adjustedFormat));
+  localMedia->AddSDPMediaFormat(new SDPMediaFormat(*localMedia, adjustedFormat));
 }
 
 
@@ -1127,7 +1129,7 @@ bool SIPConnection::WriteINVITE(OpalTransport & transport)
   // name hasn't been first specified by a register handler. i.e a
   // register handler's target number is always used
   PString number(m_connStringOptions("Calling-Party-Number"));
-  if (!number.IsEmpty() && myAddress.GetUserName() == endpoint.GetDefaultLocalPartyName())
+  if (!number.IsEmpty())
     myAddress.SetUserName(number);
 
   PString name(m_connStringOptions("Calling-Party-Name"));
@@ -1141,6 +1143,7 @@ bool SIPConnection::WriteINVITE(OpalTransport & transport)
   if (myAddress.GetDisplayName(false).IsEmpty())
     myAddress.SetDisplayName(GetDisplayName());
 
+  myAddress.SetTag(GetToken());
   m_dialog.SetLocalURI(myAddress);
 
   NotifyDialogState(SIPDialogNotification::Trying);
@@ -1148,8 +1151,7 @@ bool SIPConnection::WriteINVITE(OpalTransport & transport)
   needReINVITE = false;
   SIPTransaction * invite = new SIPInvite(*this, transport, OpalRTPSessionManager(*this));
 
-  if (m_appearanceCode >= 0)
-    invite->GetMIME().SetAt("Alert-Info", psprintf("<>;appearance=%u", m_appearanceCode));
+  invite->GetMIME().SetAlertInfo(m_alertInfo, m_appearanceCode);
 
   // It may happen that constructing the INVITE causes the connection
   // to be released (e.g. there are no UDP ports available for the RTP sessions)
@@ -1226,12 +1228,28 @@ PString SIPConnection::GetDestinationAddress()
 
 PString SIPConnection::GetCalledPartyURL()
 {
+  if (originalInvite != NULL)
+    return originalInvite->GetURI().AsString();
+
   SIPURL calledParty = m_dialog.GetRemoteURI();
   calledParty.Sanitise(SIPURL::ToURI);
   return calledParty.AsString();
 }
 
 
+PString SIPConnection::GetAlertingType() const
+{
+  return m_alertInfo;
+}
+
+
+bool SIPConnection::SetAlertingType(const PString & info)
+{
+  m_alertInfo = info;
+  return true;
+}
+
+
 bool SIPConnection::HoldConnection()
 {
   if (transport == NULL)
@@ -1294,6 +1312,12 @@ PString SIPConnection::GetPrefixName() const
 }
 
 
+PString SIPConnection::GetIdentifier() const
+{
+  return m_dialog.GetCallID();
+}
+
+
 PString SIPConnection::GetRemotePartyURL() const
 {
   SIPURL url = GetRemotePartyAddress();
@@ -1321,7 +1345,7 @@ void SIPConnection::OnTransactionFailed(SIPTransaction & transaction)
   if (GetPhase() >= ReleasingPhase)
     return;
 
-  bool allFailed = true;
+  bool stilltrying = false;
   {
     // The connection stays alive unless all INVITEs have failed
     PSafePtr<SIPTransaction> invitation(forkedInvitations, PSafeReference);
@@ -1330,17 +1354,25 @@ void SIPConnection::OnTransactionFailed(SIPTransaction & transaction)
         forkedInvitations.Remove(invitation++);
       else {
         if (!invitation->IsFailed())
-          allFailed = false;
+          stilltrying = true;
         ++invitation;
       }
     }
   }
 
-  // All invitations failed, die now
-  if (allFailed && GetPhase() < ConnectedPhase) {
-    releaseMethod = ReleaseWithNothing;
-    Release(EndedByConnectFail);
+  if (stilltrying || GetPhase() >= ConnectedPhase)
+    return;
+
+  // All invitations failed, die now, with correct code
+  for (PINDEX i = 0; i < PARRAYSIZE(SIPCodeToReason); i++) {
+    if (transaction.GetStatusCode() == SIPCodeToReason[i].code) {
+      SetQ931Cause(SIPCodeToReason[i].q931Cause);
+      Release(SIPCodeToReason[i].reason);
+      return;
+    }
   }
+
+  Release(EndedByConnectFail);
 }
 
 
@@ -1472,7 +1504,7 @@ void SIPConnection::NotifyDialogState(SIPDialogNotification::States state, SIPDi
   SIPDialogNotification info(url.AsString());
 
   info.m_dialogId = m_dialogNotifyId.AsString();
-  info.m_callId = GetToken();
+  info.m_callId = m_dialog.GetCallID();
 
   info.m_local.m_URI = url.AsString();
   info.m_local.m_dialogTag  = m_dialog.GetLocalTag();
@@ -1581,8 +1613,12 @@ void SIPConnection::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & r
     // Final check to see if we have forked INVITEs still running, don't
     // release connection until all of them have failed.
     for (PSafePtr<SIPTransaction> invitation(forkedInvitations, PSafeReference); invitation != NULL; ++invitation) {
-      if (invitation->IsInProgress())
+      if (invitation->IsProceeding())
         return;
+      // If we have not even got a 1xx from the remote for this forked INVITE,
+      // don't keep waiting, cancel it and take the error we got
+      if (invitation->IsTrying())
+        invitation->Cancel();
     }
   }
 
@@ -1613,12 +1649,14 @@ void SIPConnection::OnReceivedINVITE(SIP_PDU & request)
   if (IsOriginating()) {
     // Check for in the same dialog
     if (m_dialog.GetRemoteTag() != requestFromTag || m_dialog.GetLocalTag() != requestToTag) {
+      /* Weird, got incoming INVITE for a call we originated, however it is not in
+         the same dialog. Send back a refusal as this just isn't handled. */
       PTRACE(2, "SIP\tIgnoring INVITE from " << request.GetURI() << " when originated call.");
       request.SendResponse(*transport, SIP_PDU::Failure_LoopDetected);
       return;
     }
 
-    // We originated the call, so any INVITE must be a re-INVITE, 
+    // We originated the call, same dialog, so any INVITE must be a re-INVITE, 
     isReinvite = PTrue;
   }
   else {
@@ -1627,17 +1665,19 @@ void SIPConnection::OnReceivedINVITE(SIP_PDU & request)
       PTRACE(4, "SIP\tInitial INVITE from " << request.GetURI());
     }
     else {
-      // Have received multiple INVITEs, three possibilities
-      const SIPMIMEInfo & originalMIME = originalInvite->GetMIME();
-
-      // #1 - Same sequence number means it is a retransmission
-      if (originalMIME.GetCSeq() == requestMIME.GetCSeq()) {
+      /* If we have same transaction ID, it means it is a retransmission
+         of the original INVITE, probably should re-transmit last sent response
+         but we just ignore it. Still should work. */
+      if (originalInvite->GetTransactionID() == request.GetTransactionID()) {
         PTimeInterval timeSinceInvite = PTime() - originalInviteTime;
         PTRACE(3, "SIP\tIgnoring duplicate INVITE from " << request.GetURI() << " after " << timeSinceInvite);
         return;
       }
 
-      // #2 - Different "dialog" determined by the tags in the to and from fields indicate forking
+      /* Same transaction but different "dialog" determined by the tags in the to
+         and from fields probably indicate forking request or request arriving via
+         multiple IP paths. This will also include the RFC3261/8.2.2.2 case relating
+         to merged requests. */
       if (m_dialog.GetRemoteTag() != requestFromTag || m_dialog.GetLocalTag() != requestToTag) {
         PTRACE(3, "SIP\tIgnoring forked INVITE from " << request.GetURI());
         SIP_PDU response(request, SIP_PDU::Failure_LoopDetected);
@@ -1646,7 +1686,7 @@ void SIPConnection::OnReceivedINVITE(SIP_PDU & request)
         return;
       }
 
-      // #3 - A new INVITE in the same "dialog" but different cseq must be a re-INVITE
+      // Different transaction but same dialog, must be a re-INVITE
       isReinvite = PTrue;
     }
   }
@@ -1660,6 +1700,7 @@ void SIPConnection::OnReceivedINVITE(SIP_PDU & request)
   SIPMIMEInfo & mime = originalInvite->GetMIME();
 
   // update the dialog context
+  m_dialog.SetLocalTag(GetToken());
   m_dialog.Update(request);
   UpdateRemoteAddresses();
 
@@ -1670,7 +1711,7 @@ void SIPConnection::OnReceivedINVITE(SIP_PDU & request)
   }
 
   NotifyDialogState(SIPDialogNotification::Trying);
-  ExtractAppearanceCode(request);
+  mime.GetAlertInfo(m_alertInfo, m_appearanceCode);
 
   // Fill in all the various connection info, note our to/from is their from/to
   mime.GetProductInfo(remoteProductInfo);
@@ -1681,7 +1722,7 @@ void SIPConnection::OnReceivedINVITE(SIP_PDU & request)
   m_calledPartyName = request.GetURI().GetUserName();
   if (!m_calledPartyName.IsEmpty() && m_calledPartyName.FindSpan("0123456789*#") == P_MAX_INDEX) {
     m_calledPartyNumber = m_calledPartyName;
-    m_calledPartyName = request.GetURI().GetDisplayName();
+    m_calledPartyName = request.GetURI().GetDisplayName(false);
   }
 
   // get the address that remote end *thinks* it is using from the Contact field
@@ -1961,28 +2002,14 @@ void SIPConnection::OnReceivedTrying(SIP_PDU & /*response*/)
 {
   PTRACE(3, "SIP\tReceived Trying response");
   NotifyDialogState(SIPDialogNotification::Proceeding);
-}
 
-
-void SIPConnection::ExtractAppearanceCode(SIP_PDU & pdu)
-{
-  PString info = pdu.GetMIME()("Alert-Info");
-  if (info.IsEmpty())
-    return;
-
-  static const char appearance1[] = ";appearance=";
-  PINDEX pos = info.Find(appearance1);
-  if (pos != P_MAX_INDEX) {
-    m_appearanceCode = info.Mid(pos+sizeof(appearance1)).AsUnsigned();
-    return;
+  if (GetPhase() < ProceedingPhase) {
+    SetPhase(ProceedingPhase);
+    OnProceeding();
   }
-
-  static const char appearance2[] = ";x-line-id";
-  pos = info.Find(appearance2);
-  if (pos != P_MAX_INDEX)
-    m_appearanceCode = info.Mid(pos+sizeof(appearance2)).AsUnsigned();
 }
 
+
 void SIPConnection::OnStartTransaction(SIPTransaction & transaction)
 {
   endpoint.OnStartTransaction(*this, transaction);
@@ -1992,7 +2019,7 @@ void SIPConnection::OnReceivedRinging(SIP_PDU & response)
 {
   PTRACE(3, "SIP\tReceived Ringing response");
 
-  ExtractAppearanceCode(response);
+  response.GetMIME().GetAlertInfo(m_alertInfo, m_appearanceCode);
 
   if (GetPhase() < AlertingPhase) {
     SetPhase(AlertingPhase);
@@ -2276,12 +2303,14 @@ bool SIPConnection::OnReceivedSDPMediaDescription(SDPSessionDescription & sdp, u
   }
 
   // Then open the streams if the direction allows
-  if ((otherSidesDir&SDPMediaDescription::SendOnly) != 0)
+  if (recvStream == NULL && (otherSidesDir&SDPMediaDescription::SendOnly) != 0)
     ownerCall.OpenSourceMediaStreams(*this, mediaType, rtpSessionId);
 
-  PSafePtr<OpalConnection> otherParty = GetOtherPartyConnection();
-  if ((otherSidesDir&SDPMediaDescription::RecvOnly) != 0 && otherParty != NULL)
-    ownerCall.OpenSourceMediaStreams(*otherParty, mediaType, rtpSessionId);
+  if (sendStream == NULL && (otherSidesDir&SDPMediaDescription::RecvOnly) != 0) {
+    PSafePtr<OpalConnection> otherParty = GetOtherPartyConnection();
+    if (otherParty != NULL)
+      ownerCall.OpenSourceMediaStreams(*otherParty, mediaType, rtpSessionId);
+  }
 
   PTRACE_IF(3, otherSidesDir == SDPMediaDescription::Inactive, "SIP\tNo streams opened as " << mediaType << " inactive");
   return true;
@@ -2353,8 +2382,8 @@ PBoolean SIPConnection::SendInviteResponse(SIP_PDU::StatusCodes code, const char
   response.GetMIME().SetProductInfo(endpoint.GetUserAgent(), GetProductInfo());
   response.SetAllow(endpoint.GetAllowedMethods());
 
-  if (m_appearanceCode >= 0 && response.GetStatusCode() == SIP_PDU::Information_Ringing)
-    response.GetMIME().SetAt("Alert-Info", psprintf("<>;appearance=%u", m_appearanceCode));
+  if (response.GetStatusCode() == SIP_PDU::Information_Ringing)
+    response.GetMIME().SetAlertInfo(m_alertInfo, m_appearanceCode);
 
   if (response.GetStatusCode() >= 200) {
     ackPacket = response;
@@ -2683,7 +2712,7 @@ PBoolean SIPConnection::OnMediaControlXML(SIP_PDU & request)
   else {
     PTRACE(3, "SIP\tPictureFastUpdate received");
     if (LockReadWrite()) {
-      OpalMediaStreamPtr encodingStream = GetMediaStream(OpalMediaType::Video(), true);
+      OpalMediaStreamPtr encodingStream = GetMediaStream(OpalMediaType::Video(), false);
       if (encodingStream == NULL){
         PTRACE(3, "SIP\tNo video stream to update");
       } else {
