@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22087 $
+ * $Revision: 22184 $
  * $Author: rjongbloed $
- * $Date: 2009-02-18 03:41:35 +0000 (Wed, 18 Feb 2009) $
+ * $Date: 2009-03-10 02:45:29 +0000 (Tue, 10 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -375,19 +375,28 @@ PBoolean SIPEndPoint::GarbageCollection()
       ++transaction;
   }
 
-  transactions.DeleteObjectsToBeRemoved();
+  bool transactionsDone = transactions.DeleteObjectsToBeRemoved();
+
 
   PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReference);
   while (handler != NULL) {
+    // If unsubscribed then we do the shut down to clean up the handler
     if (handler->GetState() == SIPHandler::Unsubscribed && handler->ShutDown())
       activeSIPHandlers.Remove(handler++);
     else
       ++handler;
   }
 
-  activeSIPHandlers.DeleteObjectsToBeRemoved();
+  bool handlersDone = activeSIPHandlers.DeleteObjectsToBeRemoved();
 
-  return OpalEndPoint::GarbageCollection();
+
+  if (!OpalEndPoint::GarbageCollection())
+    return false;
+  
+  if (m_shuttingDown)
+    return transactionsDone && handlersDone;
+
+  return true;
 }
 
 
